#include "StdAfx.h"
#include "EvImage.h"

CEvImage::CEvImage(void)
{
	m_pPxl2D = NULL;
	m_pPxlPtr = NULL;
}

CEvImage::~CEvImage(void)
{
	Free();
}

void CEvImage::Free()
{
	CImage::Destroy();

	if (m_pPxl2D)
	{
		delete[] m_pPxl2D;
		m_pPxl2D = NULL;
		m_pPxlPtr = NULL;
	}
}

void CEvImage::CreateColorTable()
{
	if ( GetBPP() == 8 )
	{
		RGBQUAD rgb[256];
		for (int i=0; i<256; i++)
		{
			rgb[i].rgbRed = rgb[i].rgbGreen = rgb[i].rgbBlue = (BYTE)i;
		}
		SetColorTable(0, 256, rgb); 
	}
}

BOOL CEvImage::Create(LONG nWidth, LONG nHeight, WORD nDepth)
{
	////////////////
	_ASSERT( (nDepth == 8) || (nDepth == 24) || (nDepth == 32) );
	////////////////
	Free();
	////////////////
	if ( CImage::Create(nWidth, nHeight, nDepth) )
	{
		CreateColorTable();
		AllocPxl2D();
		return TRUE;
	}
	////////////////
	return FALSE;
}

BOOL CEvImage::OpenFile(LPCTSTR lpszPathName)
{
	////////////////
	Free();
	////////////////
	if ( CImage::Load(lpszPathName) == S_OK )
	{
        AllocPxl2D();
		return TRUE;
	}
	////////////////
	return FALSE;
}

BOOL CEvImage::Draw(HDC hDC, int x, int y, int width, int height) const
{
	////////////////
	if ( CImage::IsNull() ) return FALSE;
	////////////////
	::SetStretchBltMode(hDC, COLORONCOLOR);
	////////////////
	return CImage::Draw(hDC, x, y, width, height);
}

/////////////////////////////
//
/////////////////////////////

BOOL CEvImage::IsValid() const
{
	return !CImage::IsNull();
}

LPBYTE CEvImage::GetPxlPtr()
{
	return m_pPxlPtr;
}

LPBYTE* CEvImage::GetPxl2D()
{
	return m_pPxl2D;
}

LPBYTE CEvImage::GetRowPtr(LONG row)
{
	_ASSERT(m_pPxl2D != NULL);
	_ASSERT(row >= 0);
	_ASSERT(row < GetHeight());
	return m_pPxl2D[row];
}

/////////////////////////////
//
/////////////////////////////

void CEvImage::AllocPxl2D()
{
	_ASSERTE( CImage::IsNull() == FALSE );

    int i;
	int pitch = CImage::GetPitch();
	int height = CImage::GetHeight();
	LPBYTE pBits = (LPBYTE)(CImage::GetBits());
	////////////////
	if (m_pPxl2D)
    {
		delete[] m_pPxl2D;
		m_pPxl2D = NULL;
    }
	////////////////
	m_pPxl2D = new LPBYTE[height];
	_ASSERTE( m_pPxl2D );
	for ( i=0; i<height; i++ )
	{
		m_pPxl2D[i] = pBits + pitch * i;
	}
	////////////////
	m_pPxlPtr = pBits;
	if ( pitch < 0 ) m_pPxlPtr = pBits + pitch * (height - 1);
	////////////////
}